@@ -157,7 +157,7 @@ impl ParagraphWriter for ParagraphWriterService {
 
 impl ParagraphWriterService {
     #[tracing::instrument(skip_all)]
-    pub fn create(config: &ParagraphConfig) -> NodeResult<ParagraphWriterService> {
+    pub fn create(config: ParagraphConfig) -> NodeResult<ParagraphWriterService> {
         let paragraph_schema = ParagraphSchema::default();
 
         fs::create_dir(&config.path)?;
@@ -180,7 +180,7 @@ impl ParagraphWriterService {
             index,
             writer,
             schema: paragraph_schema,
-            config: config.clone(),
+            config,
         })
     }
     #[tracing::instrument(skip_all)]
@@ -345,6 +345,7 @@ mod tests {
             start: 0,
             end: DOC1_P1.len() as i32,
             sentences: HashMap::new(),
+            vectorsets_sentences: HashMap::new(),
             field: "body".to_string(),
             labels: vec!["/nsfw".to_string()],
             index: 0,
@@ -358,6 +359,7 @@ mod tests {
             start: DOC1_P1.len() as i32,
             end: (DOC1_P1.len() + DOC1_P2.len()) as i32,
             sentences: HashMap::new(),
+            vectorsets_sentences: HashMap::new(),
             field: "body".to_string(),
             labels: vec!["/tantivy".to_string(), "/test".to_string(), "/label1".to_string()],
             index: 1,
@@ -371,6 +373,7 @@ mod tests {
             start: (DOC1_P1.len() + DOC1_P2.len()) as i32,
             end: (DOC1_P1.len() + DOC1_P2.len() + DOC1_P3.len()) as i32,
             sentences: HashMap::new(),
+            vectorsets_sentences: HashMap::new(),
             field: "body".to_string(),
             labels: vec!["/three".to_string(), "/label2".to_string()],
             index: 2,
@@ -394,6 +397,7 @@ mod tests {
             start: 0,
             end: DOC1_TI.len() as i32,
             sentences: HashMap::new(),
+            vectorsets_sentences: HashMap::new(),
             field: "title".to_string(),
             labels: vec!["/cool".to_string()],
             index: 3,
@@ -434,7 +438,7 @@ mod tests {
             path: dir.path().join("paragraphs"),
         };
 
-        let mut paragraph_writer_service = ParagraphWriterService::create(&psc).unwrap();
+        let mut paragraph_writer_service = ParagraphWriterService::create(psc).unwrap();
         let resource1 = create_resource("shard1".to_string());
         let _ = paragraph_writer_service.set_resource(&resource1);
         let _ = paragraph_writer_service.set_resource(&resource1);
