@@ -27,6 +27,7 @@ mod params;
 #[cfg(test)]
 mod tests;
 
+use crate::config::Similarity;
 use crate::data_types::{data_store, trie, trie_ram, vector, DeleteLog};
 use crate::formula::Formula;
 use crate::VectorR;
@@ -369,13 +370,6 @@ impl DeleteLog for NoDLog {
     }
 }
 
-#[derive(Default, Debug, Clone, Copy, Serialize, Deserialize)]
-pub enum Similarity {
-    Dot,
-    #[default]
-    Cosine,
-}
-
 #[derive(Clone, Copy, Serialize, Deserialize, Debug)]
 pub struct Journal {
     uid: DpId,
