@@ -18,7 +18,8 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
 
-use crate::data_point::{self, DataPointPin, DpId, Journal, OpenDataPoint, Similarity};
+use crate::config::{Similarity, VectorConfig};
+use crate::data_point::{self, DataPointPin, DpId, Journal, OpenDataPoint};
 use crate::data_point_provider::state::*;
 use crate::data_point_provider::state::{read_state, write_state};
 use crate::data_point_provider::TimeSensitiveDLog;
@@ -149,7 +150,7 @@ struct OnlineDataPoint {
 
 pub struct Writer {
     has_uncommitted_changes: bool,
-    metadata: IndexMetadata,
+    config: VectorConfig,
     path: PathBuf,
     added_data_points: Vec<DataPointPin>,
     added_to_delete_log: Vec<(Vec<u8>, SystemTime)>,
@@ -194,8 +195,14 @@ impl Writer {
         let mut nodes_in_merge = 0;
         let mut inputs = Vec::new();
 
-        // Order smallest segments last (first to be pop()), so they are merged first
-        live_segments.sort_unstable_by_key(|i| std::cmp::Reverse(i.journal.no_nodes()));
+        let pruning_deletions = self.delete_log.size() > parameters.maximum_deleted_entries;
+        if pruning_deletions {
+            // Too many deletins, order oldest segments last (first to pop()) so we can prune the delete log
+            live_segments.sort_unstable_by_key(|i| std::cmp::Reverse(i.journal.time()));
+        } else {
+            // Order smallest segments last (first to be pop()), so they are merged first
+            live_segments.sort_unstable_by_key(|i| std::cmp::Reverse(i.journal.no_nodes()));
+        }
 
         let dtrie_copy = self.delete_log.clone();
         while nodes_in_merge < parameters.max_nodes_in_merge {
@@ -204,7 +211,7 @@ impl Writer {
             };
             let data_point_size = online_data_point.journal.no_nodes();
 
-            if data_point_size + nodes_in_merge > parameters.max_nodes_in_merge {
+            if data_point_size + nodes_in_merge > parameters.max_nodes_in_merge && nodes_in_merge > 0 {
                 break;
             }
 
@@ -213,7 +220,8 @@ impl Writer {
             nodes_in_merge += data_point_size;
         }
 
-        if inputs.len() < 2 {
+        // We need at least one segment for pruning deletes and 2 for an actual merge
+        if inputs.is_empty() || (inputs.len() < 2 && !pruning_deletions) {
             return Ok(None);
         }
 
@@ -222,7 +230,7 @@ impl Writer {
             dtrie_copy,
             destination: Some(destination),
             inputs,
-            similarity: self.metadata.similarity,
+            similarity: self.config.similarity,
             segments_left: live_segments.len() + 1,
             merge_time: SystemTime::now(),
         })))
@@ -341,7 +349,7 @@ impl Writer {
         Ok(())
     }
 
-    pub fn new(path: &Path, metadata: IndexMetadata, shard_id: String) -> VectorR<Writer> {
+    pub fn new(path: &Path, config: VectorConfig, shard_id: String) -> VectorR<Writer> {
         std::fs::create_dir(path)?;
         File::create(path.join(OPENING_FLAG))?;
 
@@ -352,11 +360,11 @@ impl Writer {
             return Err(VectorErr::MultipleWritersError);
         }
 
-        metadata.write(path)?;
+        IndexMetadata::write(&config, path)?;
         persist_state(path, &State::default())?;
 
         Ok(Writer {
-            metadata,
+            config,
             path: path.to_path_buf(),
             added_data_points: Vec::new(),
             added_to_delete_log: Vec::new(),
@@ -382,11 +390,11 @@ impl Writer {
         let lock_file = File::create(lock_path)?;
         lock_file.lock_shared()?;
 
-        let metadata = IndexMetadata::open(path)?.map(Ok).unwrap_or_else(|| {
+        let config = IndexMetadata::open(path)?.map(Ok).unwrap_or_else(|| {
             // Old indexes may not have this file so in that case the
             // metadata file they should have is created.
-            let metadata = IndexMetadata::default();
-            metadata.write(path).map(|_| metadata)
+            let config = VectorConfig::default();
+            IndexMetadata::write(&config, path).map(|_| config)
         })?;
 
         let state_path = path.join(STATE);
@@ -418,7 +426,7 @@ impl Writer {
         }
 
         Ok(Writer {
-            metadata,
+            config,
             online_data_points,
             delete_log,
             dimension,
@@ -478,8 +486,8 @@ impl Writer {
         &self.path
     }
 
-    pub fn metadata(&self) -> &IndexMetadata {
-        &self.metadata
+    pub fn config(&self) -> &VectorConfig {
+        &self.config
     }
 
     pub fn size(&self) -> usize {
@@ -490,8 +498,9 @@ impl Writer {
 #[cfg(test)]
 mod test {
 
+    use std::time::Duration;
+
     use super::*;
-    use crate::data_point::Similarity;
     use data_point::create;
 
     #[test]
@@ -499,11 +508,12 @@ mod test {
         let dir = tempfile::tempdir().unwrap();
         let vectors_path = dir.path().join("vectors");
 
-        let mut writer = Writer::new(&vectors_path, IndexMetadata::default(), "abc".into()).unwrap();
+        let mut writer = Writer::new(&vectors_path, VectorConfig::default(), "abc".into()).unwrap();
         let mut data_points = vec![];
         let merge_parameters = MergeParameters {
             segments_before_merge: 10,
             max_nodes_in_merge: 50_000,
+            maximum_deleted_entries: 25_000,
         };
 
         for _ in 0..100 {
@@ -533,11 +543,12 @@ mod test {
         let dir = tempfile::tempdir().unwrap();
         let vectors_path = dir.path().join("vectors");
 
-        let mut writer = Writer::new(&vectors_path, IndexMetadata::default(), "abc".into()).unwrap();
+        let mut writer = Writer::new(&vectors_path, VectorConfig::default(), "abc".into()).unwrap();
         let mut data_points = vec![];
         let merge_parameters = MergeParameters {
             segments_before_merge: 1000,
             max_nodes_in_merge: 50_000,
+            maximum_deleted_entries: 25_000,
         };
 
         for _ in 0..50 {
@@ -565,11 +576,12 @@ mod test {
         let dir = tempfile::tempdir().unwrap();
         let vectors_path = dir.path().join("vectors");
 
-        let mut writer = Writer::new(&vectors_path, IndexMetadata::default(), "abc".into()).unwrap();
+        let mut writer = Writer::new(&vectors_path, VectorConfig::default(), "abc".into()).unwrap();
         let mut data_points = vec![];
         let merge_parameters = MergeParameters {
             segments_before_merge: 100,
             max_nodes_in_merge: 50_000,
+            maximum_deleted_entries: 25_000,
         };
 
         for _ in 0..100 {
@@ -597,4 +609,51 @@ mod test {
         let merge_proposal = writer.prepare_merge(merge_parameters).unwrap();
         assert!(merge_proposal.is_none());
     }
+
+    #[test]
+    fn merge_old_segments() {
+        let dir = tempfile::tempdir().unwrap();
+        let vectors_path = dir.path().join("vectors");
+
+        // Writer with a single empty vector
+        let mut writer = Writer::new(&vectors_path, VectorConfig::default(), "abc".into()).unwrap();
+        let merge_parameters = MergeParameters {
+            segments_before_merge: 1,
+            max_nodes_in_merge: 50_000,
+            maximum_deleted_entries: 10,
+        };
+
+        let similarity = Similarity::Cosine;
+        let embeddings = vec![];
+        let time = Some(SystemTime::now());
+        let data_point_pin = DataPointPin::create_pin(&vectors_path).unwrap();
+        create(&data_point_pin, embeddings, time, similarity).unwrap();
+
+        let online_data_point = OnlineDataPoint {
+            journal: data_point_pin.read_journal().unwrap(),
+            pin: data_point_pin,
+        };
+        writer.online_data_points = vec![online_data_point];
+
+        // Should not merge, there is no reason to with a single segment
+        assert!(writer.prepare_merge(merge_parameters).unwrap().is_none());
+
+        // Create a bunch of deletions
+        let past = SystemTime::now().checked_sub(Duration::from_secs(5)).unwrap();
+        for i in 0..20 {
+            writer.record_delete(format!("delete{i:02}").as_bytes(), past);
+        }
+        writer.commit().unwrap();
+        assert_eq!(writer.delete_log.size(), 20);
+
+        // Should merge, just to prune the deletion tree
+        let result = writer.prepare_merge(merge_parameters).unwrap().unwrap().run().unwrap();
+        writer.record_merge(result.as_ref()).unwrap();
+
+        assert_eq!(writer.delete_log.size(), 0);
+
+        // Nothing else to merge
+        let merge_proposal = writer.prepare_merge(merge_parameters).unwrap();
+        assert!(merge_proposal.is_none());
+    }
 }
